/*
 * File name: prg_serial_winapi.c
 * Date:      2017/04/19 23:55
 * Author:    Pavel Paces
 */
#include "stdafx.h"
#include "prg_serial_winapi.h"

#ifndef BAUD_RATE
#define BAUD_RATE B115200
#endif

/// ----------------------------------------------------------------------------
HANDLE serialWinApi_open( LPCTSTR lpszDevice )
{
   DWORD dwInQueue = 16;
   DWORD dwOutQueue = 16;

   // Open the device
   HANDLE hFile = CreateFile(lpszDevice,
         GENERIC_READ|GENERIC_WRITE,
         0,
         0,
         OPEN_EXISTING,
         0,
         0);

   if (hFile == INVALID_HANDLE_VALUE)
   {
      return NULL;
   }

   if (!SetupComm( hFile, dwInQueue, dwOutQueue))
   {
      // error
      (void)serialWinApi_close( hFile );
      return NULL;
   }

   // Obtain the DCB structure for the device
   {
      DCB dcb;
      if (!GetCommState(hFile,&dcb))
      {
         // error
         (void)serialWinApi_close( hFile );
         return NULL;
      }

      // Set the new data
      dcb.BaudRate = (DWORD)(CBR_115200);
      dcb.ByteSize = (BYTE)(8);
      dcb.Parity   = (BYTE)(NOPARITY);
      dcb.StopBits = (BYTE)(ONESTOPBIT);

      // Set the new DCB structure
      if (!SetCommState(hFile,&dcb))
      {
         // Obtain the error code
         (void)serialWinApi_close( hFile );
         return NULL;
      }
   }

   // Return successful
   return hFile;
}

/// ----------------------------------------------------------------------------
int serialWinApi_close( HANDLE hSerial )
{
   // Close COM port
   CloseHandle( hSerial );
   return 1;
}

/// ----------------------------------------------------------------------------
int serialWinApi_putc( HANDLE hSerial, char c)
{
   DWORD dwWritten;
   DWORD dwRetVal;
   // Write the data
   dwRetVal = WriteFile(hSerial, &c, 1, &dwWritten, 0); 
   return dwWritten;
}

/// ----------------------------------------------------------------------------
int serialWinApi_getc( HANDLE hSerial, char * c)
{

   DWORD dwRead;
   DWORD dwRetVal;
   // Read the data
   dwRetVal = ReadFile(hSerial, c, 1, &dwRead, 0);
   return dwRetVal;
}

